# -*- coding: utf_8 -*-
from django import forms

from .models import Participantes, Time, Pessoa, Experiencia, PeriodoPessoa


# https://docs.djangoproject.com/en/1.9/ref/forms/validation/
# http://stackoverflow.com/questions/3853657/django-manytomany-validation
class MyParticipantesForm(forms.ModelForm):
    class Meta:
        model = Participantes
        fields = ['atividade', 'pessoas']

    def clean(self):
        cleaned_data = super(MyParticipantesForm, self).clean()

        atividade = cleaned_data.get("atividade", None)
        try:
            pessoas_todas = set(cleaned_data.get("pessoas", None))
        except TypeError:
            return

        # Treat case in which there is no Participantes
        # before this point (E.g. Creation of new Atividade)
        try:
            pessoas_antes = set(Participantes.objects.get(atividade=atividade).pessoas.all())
        except Participantes.DoesNotExist:
            return

        pessoas_novas = pessoas_todas - pessoas_antes

        for pn in pessoas_novas:
            pn_has_group = False

            pn_tags = pn.tags.all()
            for tag in pn_tags:
                if tag.nome.startswith('!'):
                    pn_has_group = True
                    break

            for pa in pessoas_antes:
                # This verification makes sure that two different people with
                # the same user are not added to 'Participantes'
                if pn.user == pa.user:
                    raise forms.ValidationError(
                        "Não é permitido adicionar duas pessoas com o mesmo usuário!"
                    )

                pa_has_group = False
                pa_tags = pa.tags.all()
                for tag in pa_tags:
                    if tag.nome.startswith('!'):
                        pa_has_group = True
                        break

                # Check if new person has group tag and 
                # if person is already in the group. If has
                # group tag but is not inserted yet, insert it.
                # Otherwise, do nothing.
                if pn_has_group:
                    for tag in pn_tags:
                        if tag.nome.startswith('!'):
                            group = tag.nome[1:]

                            try:
                                pn.time_set.get(nome=group)
                            except Time.DoesNotExist:
                                time = Time.objects.get(nome=group)
                                pn.time_set.add(time)
                            break

                # This verification assures that 'Participantes' only has
                # all users with group associated or all user withou group.
                if pn_has_group != pa_has_group:
                    if pn_has_group:
                        raise forms.ValidationError(
                                "Não é permitido adicionar usuário com grupo em participantes sem grupos!"
                                )
                    else:
                        raise forms.ValidationError(
                                "Não é permitido adicionar usuário sem grupo em participantes com grupos!"
                                )

        # Check if people being removed from 'Participantes' have
        # group, and if so, they are removed from the group.
        pessoas_retiradas = pessoas_antes.difference(pessoas_todas)
        if pessoas_retiradas:
            for pr in pessoas_retiradas:
                pr_tags = pr.tags.all()

                group = None
                for tag in pr_tags:
                    if tag.nome.startswith('!'):
                        group = tag.nome[1:]
                        tag.pessoa_set.remove(pr)
                        break

                if group:
                    time = Time.objects.get(nome=group)
                    pr.time_set.remove(time)


class MyPessoaForm(forms.ModelForm):
    class Meta:
        model = Pessoa
        fields = ['id', 'user', 'tags', 'regioes']

    def clean(self):
        cleaned_data = super(MyPessoaForm, self).clean()

        pessoa_id = cleaned_data.get("id", None)
        pessoa = Pessoa.objects.get(id=pessoa_id)

        try:
            tags_todas = set(cleaned_data.get("tags", None))
            #print 'Tags_todas is %s' % (tags_todas)
        except TypeError:
            return

        #
        try:
            tags_antes = set(pessoa.tags.all())
            #print 'Tags_antes is %s' % (tags_antes)
        except Pessoa.DoesNotExist:
            return

        # If new tag has been included.
        if len(tags_todas) > len(tags_antes):
            tags_novas = tags_todas - tags_antes

            # check if new tag is of group
            for tag in tags_novas:
                if tag.nome.startswith('!'):
                    group = tag.nome[1:]

                    try:
                        time = Time.objects.get(nome=group)
                        time.pessoas.add(pessoa)
                        #print '%s foi inserido no time %s' % (pessoa, time.nome)
                    except Time.DoesNotExist:
                        raise forms.ValidationError(
                            "Time inexistente. Crie-o primeiro antes de inserir pessoas!"
                        )
                    #atividade = pessoa.participantes_set.get().atividade

        # If tag has been excluded.
        elif len(tags_todas) < len(tags_antes):
            tags_removidas = tags_antes - tags_todas
            #print 'tags_removidas is %s' % tags_removidas

            for tag in tags_removidas:
                if tag.nome.startswith('!'):
                    group = tag.nome[1:]

                    try:
                        time = Time.objects.get(nome=group)
                        experiencia = time.experiencia

                        # If Pessoa in Participantes, can't remove from
                        # Time and, thus, can't remove Tag.
                        if pessoa in time.participantes.pessoas.all():
                            raise forms.ValidationError("Não é permitido remover pessoa de Time caso seja Participante. Remova-o de Participantes e tente novamente.")

                    except Time.DoesNotExist:
                        break

                    # otherwise, remove tag but also remove person
                    # from Participantes of Atividade.
                    time.pessoas.remove(pessoa)
                    time.participantes.pessoas.remove(pessoa)

        # If there's been exclusion and inclusion (zero-sum).
        else:
            tags_anteriores = tags_antes.difference(tags_todas)
            tags_posteriores = tags_todas.difference(tags_antes)
            #print 'tags_anteriores is %s' % tags_anteriores
            #print 'tags_posteriores is %s' % tags_posteriores

            time_antigo = None
            for tag in tags_anteriores:
                if tag.nome.startswith('!'):
                    time_antigo = tag.nome[1:]
                    break

            time_novo = None
            for tag in tags_posteriores:
                if tag.nome.startswith('!'):
                    time_novo = tag.nome[1:]
                    break

            # remove from old, add in new team.
            if time_antigo and time_novo:
                try:
                    antigo = Time.objects.get(nome=time_antigo)
                    novo = Time.objects.get(nome=time_novo)
                except Time.DoesNotExist:
                    raise forms.ValidationError(
                        "Time inexistente. Crie-o primeiro antes de inserir pessoas!"
                    )

                antigo.pessoas.remove(pessoa)
                novo.pessoas.add(pessoa)

            # remove from old team only.
            elif time_antigo:
                try:
                    antigo = Time.objects.get(nome=time_antigo)
                except Time.DoesNotExist:
                    raise forms.ValidationError(
                        "Time inexistente."
                    )

                antigo.pessoas.remove(pessoa)

            # add to new team only.
            elif time_novo:
                try:
                    novo = Time.objects.get(nome=time_novo)
                except Time.DoesNotExist:
                    raise forms.ValidationError(
                        "Time inexistente. Crie-o primeiro antes de inserir pessoas!"
                    )

                novo.pessoas.add(pessoa)

            # do nothing.
            else:
                pass


class MyTimeForm(forms.ModelForm):
    class Meta:
        model = Time
        fields = ['nome', 'participantes', 'pessoas', 'experiencia']

    def clean(self):
        cleaned_data = super(MyTimeForm, self).clean()

        nome = cleaned_data.get("nome", None)
        participantes = cleaned_data.get("participantes", None)
        experiencia = cleaned_data.get("experiencia", None)

        if experiencia.status == 'andamento':
            raise forms.ValidationError(
                "Não é permitido modificação em Time cuja Experiência esteja em andamento ou encerrada."
            )

        # verificar se cada pessoa tem a tag de grupo para
        # ser incluída no time; e se a pessoa já possui uma
        # outra tag de time, o que caracterizaria uma
        # duplicação.

        try:
            time = Time.objects.get(nome=nome)
        except Time.DoesNotExist:
            time = None

        try:
            pessoas_todas = set(cleaned_data.get("pessoas", None))
        except TypeError:
            return

        # Treat case in which there is no Pessoas
        # before this point (E.g. Creation of new Time)
        try:
            pessoas_antes = set(Time.objects.get(nome=nome).pessoas.all())
        except Time.DoesNotExist:
            pessoas_antes = set()

        pessoas_novas = pessoas_todas - pessoas_antes

        for pessoa in pessoas_novas:
            pn_has_group = False

            nome_time = None
            for tag in pessoa.tags.all():
                if tag.nome.startswith('!') and not nome_time:
                    # Save the Time tag name to keep on
                    # parsing, to make sure that Pessoa
                    # is not in another Time already.
                    nome_time = tag.nome[1:]
                    continue
                else:
                    raise forms.ValidationError(
                        "Pessoa possui mais de uma tag de Time!"
                    )

                if nome_time == nome:
                    if not time:
                        time = Time.objects.create(nome=nome, participantes=participantes, pessoas=pessoa, experiencia=experiencia)
                    time.pessoas.add(pessoa)
                    pn_has_group = True

            if not pn_has_group:
                raise forms.ValidationError(
                    "Não é permitido adicionar Pessoa em Time sem tag de grupo!"
                )


class MyExperienciaForm(forms.ModelForm):
    class Meta:
        model = Experiencia
        fields = ['atividade', 'periodo_inicio', 'periodo_fim', 'status']

    def clean(self):
        cleaned_data = super(MyExperienciaForm, self).clean()

        atividade = cleaned_data.get("atividade", None)
        status = cleaned_data.get("status", None)

        # check if these is another Experiencia whose
        # status is 'andamento' and, if so, don't allow
        # the creation of a new one.
        if status == 'andamento':
            experiencias = Experiencia.objects.filter(atividade=atividade, status=status)

            if experiencias and len(experiencias) > 1:
                raise forms.ValidationError(
                    "Só é permitido uma Experiência com status 'Em andamento' por Atividade."
                )

        # If the status is 'andamento' or 'encerrado',
        # restrict the user from make new modifications.
        #if status != 'criado':
        #    raise forms.ValidationError(
        #        "Não é permitido modificação em Experiência em andamento ou encerrada."
        #    )


class MyPeriodoPessoaForm(forms.ModelForm):
    class Meta:
        model = PeriodoPessoa
        fields = ['time', 'pessoa', 'data_inicio', 'data_fim']

    def clean(self):
        cleaned_data = super(MyPeriodoPessoaForm, self).clean()

        time = cleaned_data.get("time", None)
        experiencia = time.experiencia

        if experiencia.status == 'andamento' or experiencia.status == 'encerrada':
            raise forms.ValidationError(
                "Não é permitido modificação em PeriodoPessoa cuja Experiência esteja em andamento ou encerrada."
            )

