# -*- coding: utf_8 -*-
from datetime import timedelta

import requests

from django.db import models
from django.contrib.auth.models import User


ORDENACAO_CHOICES = (
    ('tag', 'Tag'),
    ('autor', 'Autor'),
    ('time', 'Time'),
    ('data', 'Data'),
)
STATUS_CHOICES = (
    ('criado', 'Criado'),
    ('andamento', 'Em andamento'),
    ('encerrado', 'Encerrado'),
)


class Texto(models.Model):
    titulo = models.CharField(max_length=200)
    conteudo = models.TextField()

    class Meta:
        ordering = ['-id']

    def __unicode__(self):
        return self.titulo


class Regiao(models.Model):
    nome = models.CharField(max_length=200)
    latitude = models.CharField(max_length=200)
    longitude = models.CharField(max_length=200)
    raio = models.IntegerField(help_text='metro(s)')

    class Meta:
        ordering = ['nome']
        verbose_name_plural = 'Regiões'

    def __unicode__(self):
        return self.nome


class Tag(models.Model):
    nome = models.CharField(max_length=200)

    def __unicode__(self):
        return self.nome

    class Meta:
        ordering = ['nome']


class Tema(models.Model):
    texto = models.ForeignKey(Texto, blank=True, null=True)
    tags = models.ManyToManyField(Tag)
    regioes = models.ManyToManyField(Regiao, blank=True)

    class Meta:
        ordering = ['-id']

    def __unicode__(self):
        if self.texto:
            return self.texto.titulo
        else:
            return 'Sem Texto'


class Atividade(models.Model):
    apelido = models.CharField('nome da atividade', max_length=200)
    tema = models.ForeignKey(Tema, blank=True, null=True)
    tamanho_time = models.IntegerField('compartilhamento', default=1) # Participantes
    tags_sequenciais = models.IntegerField(default=0)
    tags_localizaveis = models.IntegerField(default=0)
    inclusao_usuario = models.IntegerField(default=0)
    tempo_total = models.DurationField(
            default=timedelta(seconds=0),
            help_text='Preencha o total em segundos (Ex.: 3600) ou use o formato DD HH:MM:SS (DD é opcional).')
    tempo_participante = models.DurationField( # 0 significa infinito
            default=timedelta(seconds=0),
            help_text='Preencha o total em segundos (Ex.: 3600) ou use o formato DD HH:MM:SS (DD é opcional).')
    tempo_som = models.DurationField(
            'som tempo',
            default=timedelta(seconds=0),
            help_text='Preencha o total em segundos (Ex.: 3600) ou use o formato DD HH:MM:SS (DD é opcional).')
    tags_selecionaveis = models.IntegerField(default=0, help_text='Máximo')
    tags_por_foto = models.IntegerField('tags max por foto', default=0, help_text='Máximo')
    criterio_ordenacao = models.CharField(max_length=200, choices=ORDENACAO_CHOICES, blank=True)
    modo_verboso = models.IntegerField('nível verboso', default=0)
    url_experiencia = models.EmailField('experiência', blank=True, help_text='Endereço de email válido para recepção conteúdo de experiência.')
    url_dinamica = models.URLField('fotos', blank=True, help_text='URL de Rest endpoint da aplicação Fotos (Exemplo: http://example.com/api/pictures/).')

    class Meta:
        ordering = ['-id']

    def __unicode__(self):
        return self.apelido


class Pessoa(models.Model):
    id = models.CharField(max_length=200, primary_key=True)
    user = models.ForeignKey(User)
    tags = models.ManyToManyField(
            Tag,
            blank=True,
            help_text='Use <strong>!*</strong> para seleção automática do nome do time e <strong>!nome-do-time</strong> para seleção manual.<br />')
    regioes = models.ManyToManyField(Regiao, blank=True)

    class Meta:
        ordering = ['id']

    def __unicode__(self):
        return self.id


class Participantes(models.Model):
    atividade = models.ForeignKey(Atividade)
    pessoas = models.ManyToManyField(Pessoa)

    class Meta:
        ordering = ['-id']
        verbose_name = 'Participantes'
        verbose_name_plural = 'Participantes'

    def __unicode__(self):
        return self.atividade.apelido


class Experiencia(models.Model):
    atividade = models.ForeignKey(Atividade)
    periodo_inicio = models.DateTimeField()
    periodo_fim = models.DateTimeField()
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='criado')

    class Meta:
        ordering = ['-id']

    def __unicode__(self):
        return self.atividade.apelido


class Time(models.Model):
    nome = models.CharField(max_length=200)
    participantes = models.ForeignKey(Participantes)
    pessoas = models.ManyToManyField(Pessoa, blank=True)
    experiencia = models.ForeignKey(Experiencia)

    class Meta:
        ordering = ['-id']

    def __unicode__(self):
        return self.nome + ' (' + str(self.experiencia.periodo_inicio)  + ')'


class PeriodoPessoa(models.Model):
    time = models.ForeignKey(Time)
    pessoa = models.ForeignKey(Pessoa)
    data_inicio = models.DateTimeField()
    data_fim = models.DateTimeField()

    class Meta:
        ordering = ['-id']

    def __unicode__(self):
        return self.pessoa.user.username + "(" + self.time.experiencia.atividade.apelido + ")"


class Foto(models.Model):
    pessoa = models.ForeignKey(Pessoa)
    data = models.DateTimeField()
    latitude = models.CharField(max_length=200)
    longitude = models.CharField(max_length=200)
    experiencia = models.ForeignKey(Experiencia)
    tags = models.ManyToManyField(Tag, blank=True)
    imagem = models.ImageField(upload_to='fotos')
    som = models.FileField(upload_to='sons', blank=True)

    # https://docs.djangoproject.com/en/1.9/topics/db/models/#overriding-predefined-model-methods
    #
    # Override the save() method, to send the photo and other data,
    # if the 'url_dinamica' attribute of class 'Atividade' is defined.
    #
    def save(self, *args, **kwargs):
        # do_something()
        super(Foto, self).save(*args, **kwargs) # call the "real" save method.

        # Check if 'url_dinamica' is set...
        url = self.experiencia.atividade.url_dinamica
        if url:
            username = self.pessoa.user.username
            password = self.pessoa.user.password
            pub_date = self.data.isoformat()
            longitude = self.longitude
            latitude = self.latitude

            tags = list()
            all_tags = self.tags.values_list()
            for tag in all_tags:
                tags.append(tag[1])
            #print tags

            img_file = {'image': (self.imagem.name, open(self.imagem.path, 'rb'))}

            try:
                # http://docs.python-requests.org/en/master/user/quickstart/#more-complicated-post-requests
                # http://docs.python-requests.org/en/master/user/quickstart/#post-a-multipart-encoded-file
                payload = {'username': username, 'password': password, 'pub_date': pub_date, 'longitude': longitude, 'latitude': latitude, 'tags': tags}
                r = requests.post(url, data=payload, files=img_file)
            except requests.exceptions.RequestException as e:
                print e

    class Meta:
        ordering = ['-id']

    def __unicode__(self):
        return self.pessoa.user.username
