# -*- coding: utf_8 -*-
import socket
import sys
from uuid import uuid4

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.shortcuts import render, redirect
from django.utils import timezone

from rest_framework import viewsets
from rest_framework import status
from rest_framework.authentication import BasicAuthentication
from rest_framework.decorators import api_view # ping
from rest_framework.decorators import permission_classes
from rest_framework.decorators import authentication_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from revelar.serializers import *
from revelar.models import *



@api_view(['GET'])
def ping(request):
    content = status.HTTP_200_OK
    return Response(content, status=status.HTTP_200_OK)



# AUTHENTICATION
@api_view(['GET'])
@authentication_classes((BasicAuthentication,))
@permission_classes((IsAuthenticated,))
def sign_in(request, apelido):
    user = User.objects.get(username=request.user)
    atividade = Atividade.objects.get(apelido=apelido)
    participantes = Participantes.objects.get(atividade=atividade)

    is_participant = False
    for p in participantes.pessoas.iterator():
        if user == p.user:
            is_participant = True
            break

    # Caso o USUÁRIO não seja participante da atividade,
    # verifica se pode incluí-lo, caso contrário,
    # retorna mensagem de erro.
    if not is_participant:

        # Verifica se atividade está configurada para
        # aceitar novos participantes.
        if atividade.inclusao_usuario:
            nome = '{0}-{1}'.format(user.username.split('@')[0], apelido)
            nome = nome.replace(' ', '').replace('_', '')

            try:
                # Caso a pessoa tenha sido criada manualmente no admin.
                pessoa = Pessoa.objects.get(id=nome)
            except Pessoa.DoesNotExist:
                pessoa = Pessoa.objects.create(id=nome, user=user)

            participantes.pessoas.add(pessoa)

            # A princípio, não necessita de tratamento,
            # pois o aplicativo só vai retornar atividades
            # em andamento.
            experiencia = Experiencia.objects.filter(atividade=atividade).get(status='andamento')
            # Create 'time' and add 'pessoa'.
            time = Time.objects.create(nome=uuid4(), participantes=participantes, experiencia=experiencia)
            time.pessoas.add(pessoa)

            return Response(UserSerializer(user, context={'request': request}).data)

        # O USUÁRIO não é participante e não pode ser incluído.
        else:
            return Response('Usuário não é participante e atividade não permite inclusão.')

    # Se o USUÁRIO já é participante da atividade, é só
    # retornar seus dados.
    else:
        return Response(UserSerializer(user, context={'request': request}).data)



# http://stackoverflow.com/q/30085996
# http://stackoverflow.com/q/16857450 (*)
@api_view(['POST'])
def sign_up(request):
    serializer = UserSerializer(data=request.data, context={'request': request})

    if serializer.is_valid():
        try:
            user = User.objects.get(username=serializer.initial_data['username'])
        except ObjectDoesNotExist:
            user = User.objects.create_user(
                email=serializer.initial_data['username'],
                username=serializer.initial_data['username'],
                password=serializer.initial_data['password'],
            )

        email = serializer.initial_data['username']
        atividade = request.data['atividade']
        nome = '{0}-{1}'.format(email.split('@')[0], atividade)
        nome = nome.replace(' ', '').replace('_', '')
        pessoa = Pessoa.objects.create(id=nome, user=user)

        # Add Pessoa to Atividade
        participantes = Participantes.objects.get(atividade__apelido=atividade)
        participantes.pessoas.add(pessoa)

        # 2016-05-11
        experiencia = Experiencia.objects.filter(atividade__apelido=atividade).get(status='andamento')
        time = Time.objects.create(nome=uuid4(),
                                   participantes=participantes,
                                   experiencia=experiencia)
        time.pessoas.add(pessoa)

        return Response(UserSerializer(user, context={'request': request}).data)
    else:
        return Response(serializer.errors)



@api_view(['POST'])
def usuarios_de_time(request):
    usuario = request.data['usuario']
    atividade = request.data['atividade']

    try:
        user = User.objects.get(username=usuario)
        time = Time.objects.filter(experiencia__atividade__apelido=atividade).filter(pessoas__user__username=usuario)[0]
        pessoas = time.pessoas.all()
    except IndexError:
        pessoas = []

    serializer = PessoaSerializer(pessoas, many=True, context={'request': request})
    return Response(serializer.data)



@api_view(['GET'])
def experiencia_em_andamento(request, atividade_id):
    try:
        experiencia = Experiencia.objects.get(atividade__pk=atividade_id, status='andamento')
    except Experiencia.DoesNotExist:
        experiencia = None

    serializer = ExperienciaSerializer(experiencia, context={'request': request})
    return Response(serializer.data)



@api_view(['GET'])
def experiencia_encerrada(request, experiencia_id):
    try:
        experiencia = Experiencia.objects.get(pk=experiencia_id)
    except Experiencia.DoesNotExist:
        experiencia = None

    serializer = ExperienciaFinalizadaSerializer(experiencia, context={'request': request})
    return Response(serializer.data)



class TextoViewSet(viewsets.ModelViewSet):
    queryset = Texto.objects.all()
    serializer_class = TextoSerializer



class RegiaoViewSet(viewsets.ModelViewSet):
    queryset = Regiao.objects.all()
    serializer_class = RegiaoSerializer



class TagViewSet(viewsets.ModelViewSet):
    queryset = Tag.objects.all()
    serializer_class = TagSerializer

class TemaViewSet(viewsets.ModelViewSet):
    queryset = Tema.objects.all()
    serializer_class = TemaSerializer



class AtividadeViewSet(viewsets.ModelViewSet):
    queryset = Atividade.objects.all()
    serializer_class = AtividadeSerializer


    def retrieve(self, request, pk=None):
        atividade = Atividade.objects.get(pk=pk)

        try:
            # If there is an experience in progress already,
            # dont create a new one.
            experiencia = atividade.experiencia_set.get(status='andamento')
        except ObjectDoesNotExist:
            # Otherwise, create a new experience, based on the
            # total time registered in the activity.
            agora = timezone.now()
            experiencia = Experiencia(atividade=atividade,
                                      periodo_inicio=agora,
                                      periodo_fim=agora+atividade.tempo_total,
                                      status='andamento')
            experiencia.save()

            # Teams creation
            participantes = atividade.participantes_set.all()[0] # TODO
            pessoas = participantes.pessoas.all()

            for pessoa in pessoas:
                esquecido = True

                # 2016-05-26
                # Associate pessoa with certain team if there is a tag starting
                # with '!'.
                for tag in pessoa.tags.all():
                    if tag.nome.find('!') == 0 and (tag.nome.find('*') != 1):
                        nome_do_time = tag.nome.lstrip('!')
                        esquecido = False
                        # Check if the team exists...
                        try:
                            time = experiencia.time_set.all().get(nome=nome_do_time)
                            time.pessoas.add(pessoa)
                        # Otherwise, create it.
                        except ObjectDoesNotExist:
                            time = Time(nome=nome_do_time,
                                        participantes=participantes,
                                        experiencia=experiencia)
                            time.save()
                            time.pessoas.add(pessoa)
                # But if there is no such tag associated with the pessoa,
                # then he/she is associated with a team whose name is an uuid.
                if esquecido:
                    time = Time.objects.create(nome=uuid4(),
                                               participantes=participantes,
                                               experiencia=experiencia)
                    time.pessoas.add(pessoa)

        serializer = AtividadeSerializer(atividade, context={'request': request})
        return Response(serializer.data)



class UserViewSet(viewsets.ModelViewSet):
    queryset = User.objects.all().order_by('-date_joined')
    serializer_class = UserSerializer



class PessoaViewSet(viewsets.ModelViewSet):
    queryset = Pessoa.objects.all()
    serializer_class = PessoaSerializer
    # http://www.django-rest-framework.org/api-guide/routers/#simplerouter
    lookup_value_regex = '[a-zA-Z0-9-. ]+'



class ParticipantesViewSet(viewsets.ModelViewSet):
    queryset = Participantes.objects.all()
    serializer_class = ParticipantesSerializer



class ExperienciaViewSet(viewsets.ModelViewSet):
    queryset = Experiencia.objects.all()
    serializer_class = ExperienciaSerializer



class TimeViewSet(viewsets.ModelViewSet):
    queryset = Time.objects.all()
    serializer_class = TimeSerializer

    # http://stackoverflow.com/a/30409103
    # https://github.com/tomchristie/django-rest-framework/blob/master/rest_framework/mixins.py#L13
    def create(self, request):
	if request.data['nome']:
	    serializer = self.get_serializer(data=request.data)
	    serializer.is_valid(raise_exception=True)
	    self.perform_create(serializer)
	    headers = self.get_success_headers(serializer.data)
	    return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)
	else:
            # https://docs.python.org/2/library/string.html#string.split
	    experiencia_id = request.data['experiencia'].split('/')[-2]
	    experiencia = Experiencia.objects.get(pk=experiencia_id)
	    times = experiencia.time_set.all()

	    nome_maior = 0
	    for time in times:
		if nome_maior < time.nome:
		    nome_maior = time.nome
	    nome_maior = str(int(nome_maior) + 1)

	    # https://docs.djangoproject.com/en/1.9/ref/request-response/#querydict-objects
	    request.data.__setitem__('nome', nome_maior)

	    serializer = self.get_serializer(data=request.data)
	    serializer.is_valid(raise_exception=True)
	    self.perform_create(serializer)
	    headers = self.get_success_headers(serializer.data)
	    return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

        serializer = TimeSerializer(time, context={'request': request})
        return Response(serializer.data)



class PeriodoPessoaViewSet(viewsets.ModelViewSet):
    queryset = PeriodoPessoa.objects.all()
    serializer_class = PeriodoPessoaSerializer



class FotoViewSet(viewsets.ModelViewSet):
    queryset = Foto.objects.all()
    serializer_class = FotoSerializer



# INTERFACE
@staff_member_required
def index(request):
    return render(request, 'revelar/index.html', locals())


@staff_member_required
def experiencias_abertas(request):
    experiencias = Experiencia.objects.filter(status='andamento')
    return render(request, 'revelar/experiencias-abertas.html', locals())


@staff_member_required
def finalizar(request, exp_id):
    experiencia = Experiencia.objects.get(pk=exp_id)
    experiencia.status = 'encerrado'
    experiencia.periodo_fim = timezone.now()
    experiencia.save()

    if experiencia.atividade.url_experiencia:
        # send email with REST point link containing experience data
        # function: experiencia_encerrada
        send_mail(
            'Fim da atividade {0}'.format(experiencia.atividade.apelido),
            'http://{0}{1}'.format(socket.gethostname(), reverse('experiencia_encerrada', kwargs={'experiencia_id': exp_id})),
            'no-reply@visgraf.impa.br',
            [experiencia.atividade.url_experiencia],
            fail_silently=False,
        )

    messages.add_message(request, messages.SUCCESS, 'Experiencia encerrada com sucesso.')
    return redirect('experiencias_abertas')


@staff_member_required
def busca_de_foto(request):
    return render(request, 'revelar/fotos-search.html', locals())


@staff_member_required
def ver_todas(request):
    fotos_todas = Foto.objects.all()
    return render(request, 'revelar/fotos-visual.html', locals())


@staff_member_required
def lista_de_experiencias(request):
    experiencias = Experiencia.objects.filter(status='encerrado')
    return render(request, 'revelar/lista-experiencias.html', locals())


@staff_member_required
def ver_experiencia(request, exp_id):
    experiencia = Experiencia.objects.get(pk=exp_id)

    fotos = Foto.objects.filter(experiencia=experiencia)
    fotos_por_pessoa = dict()
    for foto in fotos:
	chave = foto.pessoa.user.username
	if chave not in fotos_por_pessoa.keys():
	    fotos_por_pessoa[chave] = []
	fotos_por_pessoa[chave].append(foto)

    return render(request, 'revelar/experiencia.html', locals())

